@@ -37,9 +37,6 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#include <X11/XKBlib.h>
-#include <X11/keysym.h>
-
 #include "gnome-settings-bus.h"
 #include "gnome-settings-profile.h"
 #include "gsd-keyboard-manager.h"
@@ -47,14 +44,10 @@
 #include "gsd-enums.h"
 #include "gsd-settings-migrate.h"
 
-#define GSD_KEYBOARD_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_KEYBOARD_MANAGER, GsdKeyboardManagerPrivate))
-
 #define GSD_KEYBOARD_DIR "org.gnome.settings-daemon.peripherals.keyboard"
 
 #define KEY_CLICK          "click"
 #define KEY_CLICK_VOLUME   "click-volume"
-#define KEY_REMEMBER_NUMLOCK_STATE "remember-numlock-state"
-#define KEY_NUMLOCK_STATE  "numlock-state"
 
 #define KEY_BELL_VOLUME    "bell-volume"
 #define KEY_BELL_PITCH     "bell-pitch"
@@ -78,24 +71,31 @@
 
 #define DEFAULT_LAYOUT "us"
 
-struct GsdKeyboardManagerPrivate
+#define GNOME_A11Y_APPLICATIONS_INTERFACE_DIR "org.gnome.desktop.a11y.applications"
+#define KEY_OSK_ENABLED "screen-keyboard-enabled"
+
+struct _GsdKeyboardManager
 {
-	guint      start_idle_id;
+        GObject    parent;
+
+        guint      start_idle_id;
         GSettings *settings;
         GSettings *input_sources_settings;
+        GSettings *a11y_settings;
         GDBusProxy *localed;
         GCancellable *cancellable;
 
-        gint       xkb_event_base;
-        GsdNumLockState old_state;
         GdkDeviceManager *device_manager;
         guint device_added_id;
+        guint device_removed_id;
 };
 
 static void     gsd_keyboard_manager_class_init  (GsdKeyboardManagerClass *klass);
 static void     gsd_keyboard_manager_init        (GsdKeyboardManager      *keyboard_manager);
 static void     gsd_keyboard_manager_finalize    (GObject                 *object);
 
+static void     update_gtk_im_module (GsdKeyboardManager *manager);
+
 G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
@@ -143,118 +143,6 @@ schema_is_installed (const char *schema)
         return installed;
 }
 
-static gboolean
-check_xkb_extension (GsdKeyboardManager *manager)
-{
-        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        int opcode, error_base, major, minor;
-        gboolean have_xkb;
-
-        have_xkb = XkbQueryExtension (dpy,
-                                      &opcode,
-                                      &manager->priv->xkb_event_base,
-                                      &error_base,
-                                      &major,
-                                      &minor);
-        return have_xkb;
-}
-
-static void
-xkb_init (GsdKeyboardManager *manager)
-{
-        Display *dpy;
-
-        dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        XkbSelectEventDetails (dpy,
-                               XkbUseCoreKbd,
-                               XkbStateNotify,
-                               XkbModifierLockMask,
-                               XkbModifierLockMask);
-}
-
-static unsigned
-numlock_NumLock_modifier_mask (void)
-{
-        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        return XkbKeysymToModifiers (dpy, XK_Num_Lock);
-}
-
-static void
-numlock_set_xkb_state (GsdNumLockState new_state)
-{
-        unsigned int num_mask;
-        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        if (new_state != GSD_NUM_LOCK_STATE_ON && new_state != GSD_NUM_LOCK_STATE_OFF)
-                return;
-        num_mask = numlock_NumLock_modifier_mask ();
-        XkbLockModifiers (dpy, XkbUseCoreKbd, num_mask, new_state == GSD_NUM_LOCK_STATE_ON ? num_mask : 0);
-}
-
-static const char *
-num_lock_state_to_string (GsdNumLockState numlock_state)
-{
-	switch (numlock_state) {
-	case GSD_NUM_LOCK_STATE_UNKNOWN:
-		return "GSD_NUM_LOCK_STATE_UNKNOWN";
-	case GSD_NUM_LOCK_STATE_ON:
-		return "GSD_NUM_LOCK_STATE_ON";
-	case GSD_NUM_LOCK_STATE_OFF:
-		return "GSD_NUM_LOCK_STATE_OFF";
-	default:
-		return "UNKNOWN";
-	}
-}
-
-static GdkFilterReturn
-xkb_events_filter (GdkXEvent *xev_,
-		   GdkEvent  *gdkev_,
-		   gpointer   user_data)
-{
-        XEvent *xev = (XEvent *) xev_;
-	XkbEvent *xkbev = (XkbEvent *) xev;
-        GsdKeyboardManager *manager = (GsdKeyboardManager *) user_data;
-
-        if (xev->type != manager->priv->xkb_event_base ||
-            xkbev->any.xkb_type != XkbStateNotify)
-		return GDK_FILTER_CONTINUE;
-
-	if (xkbev->state.changed & XkbModifierLockMask) {
-		unsigned num_mask = numlock_NumLock_modifier_mask ();
-		unsigned locked_mods = xkbev->state.locked_mods;
-		GsdNumLockState numlock_state;
-
-		numlock_state = (num_mask & locked_mods) ? GSD_NUM_LOCK_STATE_ON : GSD_NUM_LOCK_STATE_OFF;
-
-		if (numlock_state != manager->priv->old_state) {
-			g_debug ("New num-lock state '%s' != Old num-lock state '%s'",
-				 num_lock_state_to_string (numlock_state),
-				 num_lock_state_to_string (manager->priv->old_state));
-			g_settings_set_enum (manager->priv->settings,
-					     KEY_NUMLOCK_STATE,
-					     numlock_state);
-			manager->priv->old_state = numlock_state;
-		}
-	}
-
-        return GDK_FILTER_CONTINUE;
-}
-
-static void
-install_xkb_filter (GsdKeyboardManager *manager)
-{
-        gdk_window_add_filter (NULL,
-                               xkb_events_filter,
-                               manager);
-}
-
-static void
-remove_xkb_filter (GsdKeyboardManager *manager)
-{
-        gdk_window_remove_filter (NULL,
-                                  xkb_events_filter,
-                                  manager);
-}
-
 static void
 apply_bell (GsdKeyboardManager *manager)
 {
@@ -268,7 +156,7 @@ apply_bell (GsdKeyboardManager *manager)
         int              click_volume;
 
         g_debug ("Applying the bell settings");
-        settings      = manager->priv->settings;
+        settings      = manager->settings;
         click         = g_settings_get_boolean  (settings, KEY_CLICK);
         click_volume  = g_settings_get_int   (settings, KEY_CLICK_VOLUME);
 
@@ -298,33 +186,10 @@ apply_bell (GsdKeyboardManager *manager)
         gdk_error_trap_pop_ignored ();
 }
 
-static void
-apply_numlock (GsdKeyboardManager *manager)
-{
-	GSettings *settings;
-        gboolean rnumlock;
-
-        g_debug ("Applying the num-lock settings");
-        settings = manager->priv->settings;
-        rnumlock = g_settings_get_boolean  (settings, KEY_REMEMBER_NUMLOCK_STATE);
-        manager->priv->old_state = g_settings_get_enum (manager->priv->settings, KEY_NUMLOCK_STATE);
-
-        gdk_error_trap_push ();
-        if (rnumlock) {
-                g_debug ("Remember num-lock is set, so applying setting '%s'",
-                         num_lock_state_to_string (manager->priv->old_state));
-                numlock_set_xkb_state (manager->priv->old_state);
-        }
-
-        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
-        gdk_error_trap_pop_ignored ();
-}
-
 static void
 apply_all_settings (GsdKeyboardManager *manager)
 {
 	apply_bell (manager);
-	apply_numlock (manager);
 }
 
 static void
@@ -339,11 +204,6 @@ settings_changed (GSettings          *settings,
 	    g_strcmp0 (key, KEY_BELL_MODE) == 0) {
 		g_debug ("Bell setting '%s' changed, applying bell settings", key);
 		apply_bell (manager);
-	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0) {
-		g_debug ("Remember Num-Lock state '%s' changed, applying num-lock settings", key);
-		apply_numlock (manager);
-	} else if (g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
-		g_debug ("Num-Lock state '%s' changed, will apply at next startup", key);
 	} else if (g_strcmp0 (key, KEY_BELL_CUSTOM_FILE) == 0){
 		g_debug ("Ignoring '%s' setting change", KEY_BELL_CUSTOM_FILE);
 	} else {
@@ -360,12 +220,23 @@ device_added_cb (GdkDeviceManager   *device_manager,
         GdkInputSource source;
 
         source = gdk_device_get_source (device);
-        if (source == GDK_SOURCE_KEYBOARD) {
-                g_debug ("New keyboard plugged in, applying all settings");
-                apply_numlock (manager);
+        if (source == GDK_SOURCE_TOUCHSCREEN) {
+                update_gtk_im_module (manager);
         }
 }
 
+static void
+device_removed_cb (GdkDeviceManager   *device_manager,
+                   GdkDevice          *device,
+                   GsdKeyboardManager *manager)
+{
+        GdkInputSource source;
+
+        source = gdk_device_get_source (device);
+        if (source == GDK_SOURCE_TOUCHSCREEN)
+                update_gtk_im_module (manager);
+}
+
 static void
 set_devicepresence_handler (GsdKeyboardManager *manager)
 {
@@ -376,9 +247,11 @@ set_devicepresence_handler (GsdKeyboardManager *manager)
 
         device_manager = gdk_display_get_device_manager (gdk_display_get_default ());
 
-        manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
+        manager->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
                                                            G_CALLBACK (device_added_cb), manager);
-        manager->priv->device_manager = device_manager;
+        manager->device_removed_id = g_signal_connect (G_OBJECT (device_manager), "device-removed",
+                                                             G_CALLBACK (device_removed_cb), manager);
+        manager->device_manager = device_manager;
 }
 
 static gboolean
@@ -395,14 +268,36 @@ need_ibus (GVariant *sources)
         return FALSE;
 }
 
+static gboolean
+need_osk (GsdKeyboardManager *manager)
+{
+        gboolean has_touchscreen = FALSE;
+        GList *devices;
+        GdkSeat *seat;
+
+        if (g_settings_get_boolean (manager->a11y_settings,
+                                    KEY_OSK_ENABLED))
+                return TRUE;
+
+        seat = gdk_display_get_default_seat (gdk_display_get_default ());
+        devices = gdk_seat_get_slaves (seat, GDK_SEAT_CAPABILITY_TOUCH);
+
+        has_touchscreen = devices != NULL;
+
+        g_list_free (devices);
+
+        return has_touchscreen;
+}
+
 static void
-set_gtk_im_module (GSettings *settings,
-                   GVariant  *sources)
+set_gtk_im_module (GsdKeyboardManager *manager,
+                   GSettings          *settings,
+                   GVariant           *sources)
 {
         const gchar *new_module;
         gchar *current_module;
 
-        if (need_ibus (sources))
+        if (need_ibus (sources) || need_osk (manager))
                 new_module = GTK_IM_MODULE_IBUS;
         else
                 new_module = GTK_IM_MODULE_SIMPLE;
@@ -414,20 +309,20 @@ set_gtk_im_module (GSettings *settings,
 }
 
 static void
-input_sources_changed (GSettings          *settings,
-                       const char         *key,
-                       GsdKeyboardManager *manager)
+update_gtk_im_module (GsdKeyboardManager *manager)
 {
         GSettings *interface_settings;
         GVariant *sources;
+
         /* Gtk+ uses the IM module advertised in XSETTINGS so, if we
          * have IBus input sources, we want it to load that
          * module. Otherwise we can use the default "simple" module
          * which is builtin gtk+
          */
-        sources = g_settings_get_value (settings, KEY_INPUT_SOURCES);
         interface_settings = g_settings_new (GNOME_DESKTOP_INTERFACE_DIR);
-        set_gtk_im_module (interface_settings, sources);
+        sources = g_settings_get_value (manager->input_sources_settings,
+                                        KEY_INPUT_SOURCES);
+        set_gtk_im_module (manager, interface_settings, sources);
         g_object_unref (interface_settings);
         g_variant_unref (sources);
 }
@@ -435,15 +330,13 @@ input_sources_changed (GSettings          *settings,
 static void
 get_sources_from_xkb_config (GsdKeyboardManager *manager)
 {
-        GsdKeyboardManagerPrivate *priv = manager->priv;
         GVariantBuilder builder;
         GVariant *v;
         gint i, n;
         gchar **layouts = NULL;
         gchar **variants = NULL;
-        gboolean have_default_layout = FALSE;
 
-        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Layout");
+        v = g_dbus_proxy_get_cached_property (manager->localed, "X11Layout");
         if (v) {
                 const gchar *s = g_variant_get_string (v, NULL);
                 if (*s)
@@ -451,10 +344,14 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
                 g_variant_unref (v);
         }
 
-        if (!layouts)
-                return;
+        init_builder_with_sources (&builder, manager->input_sources_settings);
 
-        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Variant");
+        if (!layouts) {
+                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, DEFAULT_LAYOUT);
+                goto out;
+	}
+
+        v = g_dbus_proxy_get_cached_property (manager->localed, "X11Variant");
         if (v) {
                 const gchar *s = g_variant_get_string (v, NULL);
                 if (*s)
@@ -467,8 +364,6 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
         else
                 n = g_strv_length (layouts);
 
-        init_builder_with_sources (&builder, priv->input_sources_settings);
-
         for (i = 0; i < n && layouts[i][0]; ++i) {
                 gchar *id;
 
@@ -477,17 +372,12 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
                 else
                         id = g_strdup (layouts[i]);
 
-                if (g_str_equal (id, DEFAULT_LAYOUT))
-                        have_default_layout = TRUE;
-
                 g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, id);
                 g_free (id);
         }
 
-        if (!have_default_layout)
-                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, DEFAULT_LAYOUT);
-
-        g_settings_set_value (priv->input_sources_settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
+out:
+        g_settings_set_value (manager->input_sources_settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
 
         g_strfreev (layouts);
         g_strfreev (variants);
@@ -496,11 +386,10 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
 static void
 get_options_from_xkb_config (GsdKeyboardManager *manager)
 {
-        GsdKeyboardManagerPrivate *priv = manager->priv;
         GVariant *v;
         gchar **options = NULL;
 
-        v = g_dbus_proxy_get_cached_property (priv->localed, "X11Options");
+        v = g_dbus_proxy_get_cached_property (manager->localed, "X11Options");
         if (v) {
                 const gchar *s = g_variant_get_string (v, NULL);
                 if (*s)
@@ -511,7 +400,7 @@ get_options_from_xkb_config (GsdKeyboardManager *manager)
         if (!options)
                 return;
 
-        g_settings_set_strv (priv->input_sources_settings, KEY_KEYBOARD_OPTIONS, (const gchar * const*) options);
+        g_settings_set_strv (manager->input_sources_settings, KEY_KEYBOARD_OPTIONS, (const gchar * const*) options);
 
         g_strfreev (options);
 }
@@ -634,7 +523,7 @@ maybe_create_initial_settings (GsdKeyboardManager *manager)
         GVariant *sources;
         gchar **options;
 
-        settings = manager->priv->input_sources_settings;
+        settings = manager->input_sources_settings;
 
         if (g_getenv ("RUNNING_UNDER_GDM"))
                 return;
@@ -672,7 +561,7 @@ localed_proxy_ready (GObject      *source,
                 g_error_free (error);
         }
 
-        manager->priv->localed = proxy;
+        manager->localed = proxy;
         maybe_create_initial_settings (manager);
 }
 
@@ -683,17 +572,22 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
 
         g_debug ("Starting keyboard manager");
 
-        manager->priv->settings = g_settings_new (GSD_KEYBOARD_DIR);
-
-	xkb_init (manager);
+        manager->settings = g_settings_new (GSD_KEYBOARD_DIR);
 
 	set_devicepresence_handler (manager);
 
-        manager->priv->input_sources_settings = g_settings_new (GNOME_DESKTOP_INPUT_SOURCES_DIR);
-        g_signal_connect (manager->priv->input_sources_settings, "changed::"KEY_INPUT_SOURCES,
-                          G_CALLBACK (input_sources_changed), manager);
+        manager->input_sources_settings = g_settings_new (GNOME_DESKTOP_INPUT_SOURCES_DIR);
+        g_signal_connect_swapped (manager->input_sources_settings,
+                                  "changed::" KEY_INPUT_SOURCES,
+                                  G_CALLBACK (update_gtk_im_module), manager);
 
-        manager->priv->cancellable = g_cancellable_new ();
+        manager->a11y_settings = g_settings_new (GNOME_A11Y_APPLICATIONS_INTERFACE_DIR);
+        g_signal_connect_swapped (manager->a11y_settings,
+                                  "changed::" KEY_OSK_ENABLED,
+                                  G_CALLBACK (update_gtk_im_module), manager);
+        update_gtk_im_module (manager);
+
+        manager->cancellable = g_cancellable_new ();
 
         g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
                                   G_DBUS_PROXY_FLAGS_NONE,
@@ -701,7 +595,7 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                                   "org.freedesktop.locale1",
                                   "/org/freedesktop/locale1",
                                   "org.freedesktop.locale1",
-                                  manager->priv->cancellable,
+                                  manager->cancellable,
                                   localed_proxy_ready,
                                   manager);
 
@@ -710,15 +604,13 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                 g_debug ("Started the keyboard plugin, applying all settings");
                 apply_all_settings (manager);
 
-                g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
+                g_signal_connect (G_OBJECT (manager->settings), "changed",
                                   G_CALLBACK (settings_changed), manager);
         }
 
-	install_xkb_filter (manager);
-
         gnome_settings_profile_end (NULL);
 
-        manager->priv->start_idle_id = 0;
+        manager->start_idle_id = 0;
 
         return FALSE;
 }
@@ -729,13 +621,8 @@ gsd_keyboard_manager_start (GsdKeyboardManager *manager,
 {
         gnome_settings_profile_start (NULL);
 
-	if (check_xkb_extension (manager) == FALSE) {
-		g_debug ("XKB is not supported, not applying any settings");
-		return TRUE;
-	}
-
-        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_keyboard_idle_cb, manager);
-        g_source_set_name_by_id (manager->priv->start_idle_id, "[gnome-settings-daemon] start_keyboard_idle_cb");
+        manager->start_idle_id = g_idle_add ((GSourceFunc) start_keyboard_idle_cb, manager);
+        g_source_set_name_by_id (manager->start_idle_id, "[gnome-settings-daemon] start_keyboard_idle_cb");
 
         gnome_settings_profile_end (NULL);
 
@@ -745,23 +632,21 @@ gsd_keyboard_manager_start (GsdKeyboardManager *manager,
 void
 gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 {
-        GsdKeyboardManagerPrivate *p = manager->priv;
-
         g_debug ("Stopping keyboard manager");
 
-        g_cancellable_cancel (p->cancellable);
-        g_clear_object (&p->cancellable);
+        g_cancellable_cancel (manager->cancellable);
+        g_clear_object (&manager->cancellable);
 
-        g_clear_object (&p->settings);
-        g_clear_object (&p->input_sources_settings);
-        g_clear_object (&p->localed);
+        g_clear_object (&manager->settings);
+        g_clear_object (&manager->input_sources_settings);
+        g_clear_object (&manager->a11y_settings);
+        g_clear_object (&manager->localed);
 
-        if (p->device_manager != NULL) {
-                g_signal_handler_disconnect (p->device_manager, p->device_added_id);
-                p->device_manager = NULL;
+        if (manager->device_manager != NULL) {
+                g_signal_handler_disconnect (manager->device_manager, manager->device_added_id);
+                g_signal_handler_disconnect (manager->device_manager, manager->device_removed_id);
+                manager->device_manager = NULL;
         }
-
-	remove_xkb_filter (manager);
 }
 
 static void
@@ -770,14 +655,11 @@ gsd_keyboard_manager_class_init (GsdKeyboardManagerClass *klass)
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = gsd_keyboard_manager_finalize;
-
-        g_type_class_add_private (klass, sizeof (GsdKeyboardManagerPrivate));
 }
 
 static void
 gsd_keyboard_manager_init (GsdKeyboardManager *manager)
 {
-        manager->priv = GSD_KEYBOARD_MANAGER_GET_PRIVATE (manager);
 }
 
 static void
@@ -790,12 +672,12 @@ gsd_keyboard_manager_finalize (GObject *object)
 
         keyboard_manager = GSD_KEYBOARD_MANAGER (object);
 
-        g_return_if_fail (keyboard_manager->priv != NULL);
+        g_return_if_fail (keyboard_manager != NULL);
 
         gsd_keyboard_manager_stop (keyboard_manager);
 
-        if (keyboard_manager->priv->start_idle_id != 0)
-                g_source_remove (keyboard_manager->priv->start_idle_id);
+        if (keyboard_manager->start_idle_id != 0)
+                g_source_remove (keyboard_manager->start_idle_id);
 
         G_OBJECT_CLASS (gsd_keyboard_manager_parent_class)->finalize (object);
 }
@@ -804,9 +686,10 @@ static void
 migrate_keyboard_settings (void)
 {
         GsdSettingsMigrateEntry entries[] = {
-                { "repeat",          "repeat",          NULL },
-                { "repeat-interval", "repeat-interval", NULL },
-                { "delay",           "delay",           NULL }
+                { "repeat",                 "repeat",                 NULL },
+                { "repeat-interval",        "repeat-interval",        NULL },
+                { "delay",                  "delay",                  NULL },
+                { "remember-numlock-state", "remember-numlock-state", NULL },
         };
 
         gsd_settings_migrate_check ("org.gnome.settings-daemon.peripherals.keyboard.deprecated",
